#!/usr/bin/env perl

use warnings;
use strict;
use HTTP::Request::Common;
use LWP::UserAgent;
use JSON;
use Nagios::Plugin;
use Data::Dumper;
use Encode;
use HTTP::Cookies;

my $np = Nagios::Plugin->new(
    usage => "Usage: %s -u|--url <http://user:pass\@host:port/url> -a|--attributes <attributes> "
    . "[ -c|--critical <thresholds> ] [ -w|--warning <thresholds> ] "
    . "[ -I|--ZWayInstance " 
    . "[ -N|--ZWayNode (default:0)"
    . "[ -C|--ZWayCommandClass "
    . "[ -S|--ZWayScale (default:1)"
    . "[ -t|--timeout <timeout> ] "
    . "[ --ignoressl ] "
    . "[ -h|--help ] ",
    version => '0.1',
    blurb   => 'Nagios plugin to check values from Z-Way using API and Virtual devices',
    extra   => "\nExample: \n"
    . "check_zway.pl --url http://monitor:monitor\@192.168.178.10:8083"
    . "              -I 12 -C 49 -S 3 --warning :5 --critical :10 ",
    url     => 'http://www.creativeit.eu/software/nagios-plugins/check-zwaypl.html',
    plugin  => 'check_zway',
    timeout => 15,
    shortname => "CheckZWAYstatus"
);

# add valid command line options and build them into your usage/help documentation.
$np->add_arg(
    spec => 'url|u=s',
    help => '-u, --url http://user:pass@192.168.178.10:8083',
    required => 1,
);

$np->add_arg(
    spec => 'ZWayInstance|I=i',
    help => '-I, --ZWayInstance 12',
    required => 1,
);

$np->add_arg(
    spec => 'ZWayNode|N=i',
    help => '-N, --ZWayNode 0',
	default => 0,
    required => 0,
);

$np->add_arg(
    spec => 'ZWayCommandClass|C=i',
    help => '-C, --ZWayCommandClass 49',
    required => 1,
);

$np->add_arg(
    spec => 'ZWayScale|S=i',
    help => '-S, --ZWayScale 2',
	default => 0,
    required => 0,
);

$np->add_arg(
    spec => 'warning|w=s',
    help => '-w, --warning INTEGER:INTEGER . See '
    . 'http://nagiosplug.sourceforge.net/developer-guidelines.html#THRESHOLDFORMAT '
    . 'for the threshold format. ',
);

$np->add_arg(
    spec => 'critical|c=s',
    help => '-c, --critical INTEGER:INTEGER . See '
    . 'http://nagiosplug.sourceforge.net/developer-guidelines.html#THRESHOLDFORMAT '
    . 'for the threshold format. ',
);

$np->add_arg(
    spec => 'ignoressl',
    help => "--ignoressl\n   Ignore bad ssl certificates",
);


####  Parse @ARGV and process standard arguments (e.g. usage, help, version)
$np->getopts;

my $opt_zi = $np->opts->ZWayInstance;
my $opt_zn = $np->opts->ZWayNode;
my $opt_zc = $np->opts->ZWayCommandClass;
my $opt_zs = $np->opts->ZWayScale;

if ($np->opts->verbose) { print "\nVerbose: Nagios Object"; print Dumper ($np);  print "\n#----\n" };


#### Useragent
my $ua = LWP::UserAgent->new;
my $cookies = HTTP::Cookies->new( );

$ua->env_proxy;
$ua->cookie_jar( $cookies );
$ua->agent('check_json/0.5');
$ua->default_header('Accept' => 'application/json');
$ua->protocols_allowed( [ 'http', 'https'] );
$ua->parse_head(0);
$ua->timeout($np->opts->timeout);

if ($np->opts->ignoressl) {
    $ua->ssl_opts(verify_hostname => 0, SSL_verify_mode => 0x00);
}

if ($np->opts->verbose) { print "\nVerbose: Useragent"; print Dumper ($ua);  print "\n#----\n" };


#### Build URL
my $urlp = "/ZAutomation/api/v1/devices/ZWayVDev_zway_" . $opt_zi . "-" . $opt_zn . "-" . $opt_zc . "-" . $opt_zs;
##if ($opt_zs != 9999) { $urlp = $urlp . "-" . $opt_zs; }

my $url = $np->opts->url . $urlp;

if ($np->opts->verbose) { print "\nVerbose: Url"; print Dumper ($url);  print "\n#----\n" };

	
#### Get response
my $response;
$response = $ua->request(GET $url, 'Content-type' => 'application/json');

if (not $response->is_success) {
    $np->nagios_exit(CRITICAL, "Connection to " . $urlp . " failed: ".$response->status_line);
}

if ($np->opts->verbose) { print "\nVerbose: Response"; print Dumper ($response);  print "\n#----\n" };


## Parse JSON
my $json_response = decode_json($response->content);
if ($np->opts->verbose) { print "\nVerbose: JSON Response"; print Dumper ($json_response);  print "\n#----\n"};


#### Compute value and limits
my @warning = split(',', $np->opts->warning);
my @critical = split(',', $np->opts->critical);

my $check_value = $json_response->{'data'}->{'metrics'}->{'level'};
my $check_title = $json_response->{'data'}->{'metrics'}->{'title'};
my $check_probe = $json_response->{'data'}->{'metrics'}->{'probeTitle'};

# Convert scaleTitle like "\x{b0}C"
my $tmpenc = encode('UTF-8',$json_response->{'data'}->{'metrics'}->{'scaleTitle'});
my $check_scale = encode('iso-8859-1',$tmpenc);

# Check if scaleTitle is an allowed value [[u|m]s % B c}]
my $check_scale_in_list = 0;
if ($check_scale eq "s" ) { $check_scale_in_list = 1; }
if ($check_scale eq "us") { $check_scale_in_list = 1; }
if ($check_scale eq "ms") { $check_scale_in_list = 1; }
if ($check_scale eq "%" ) { $check_scale_in_list = 1; }
if ($check_scale eq "B" ) { $check_scale_in_list = 1; }
if ($check_scale eq "c" ) { $check_scale_in_list = 1; }

### Smartmatch is experimental
#if($check_scale ~~ [ "s", "us", "ms", "%", "B", "c" ] ) {
#	$check_scale_in_list = 1;
#}


# Check value against thresholds...
my $result = -1;
$result = $np->check_threshold(
    check => $check_value,
    warning => $np->opts->warning,
    critical => $np->opts->critical
 );

if ($np->opts->verbose) { 
	print "\nVerbose: Value $check_value "; 
	print "\n         Scale $check_scale"; 
	print "\n         Title $check_title"; 
	print "\n         Probe $check_probe"; 
	print "\n#----\n"
};


#### Compute value and limits

my @statusmsg;

if ($check_scale_in_list eq 1) {
	push(@statusmsg, "$check_probe: ".$check_value.$check_scale);
	$np->add_perfdata(
		label => $check_title,
		value => $check_value,
		uom => $check_scale, 
		threshold => $np->set_thresholds( warning => $np->opts->warning, critical => $np->opts->critical),
	); 
} else {
	push(@statusmsg, "$check_probe: ".$check_value);
	$np->add_perfdata(
		label => $check_title . "(" . $check_scale . ")",
		value => $check_value,
		threshold => $np->set_thresholds( warning => $np->opts->warning, critical => $np->opts->critical),
	); 
};

if ($np->opts->verbose) { print "\nVerbose: StatusMsg"; print Dumper (@statusmsg);  print "\n#----\n"};


#### Finally

$np->nagios_exit(
    return_code => $result,
    message     => join(', ', @statusmsg),
);
